package com.sun.jmx.snmp;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.Serializable;

public abstract class Enumerated implements Serializable
{
    protected int value;
    
    public Enumerated() throws IllegalArgumentException {
        Enumeration<Integer> keys = this.getIntTable().keys();
        if (keys.hasMoreElements()) {
            this.value = (int)keys.nextElement();
            return;
        }
        throw new IllegalArgumentException();
    }
    
    public Enumerated(int n) throws IllegalArgumentException {
        if (this.getIntTable().get(new Integer(n)) == null) {
            throw new IllegalArgumentException();
        }
        this.value = n;
        return;
    }
    
    public Enumerated(Integer key) throws IllegalArgumentException {
        if (this.getIntTable().get(key) == null) {
            throw new IllegalArgumentException();
        }
        this.value = (int)key;
        return;
    }
    
    public Enumerated(String key) throws IllegalArgumentException {
        Integer n = (Integer)this.getStringTable().get(key);
        if (n == null) {
            throw new IllegalArgumentException();
        }
        this.value = (int)n;
        return;
    }
    
    public int intValue() {
        return this.value;
    }
    
    public Enumeration<Integer> valueIndexes() {
        return this.getIntTable().keys();
    }
    
    public Enumeration<String> valueStrings() {
        return this.getStringTable().keys();
    }
    
    @Override
    public boolean equals(Object o) {
        return o != null && this.getClass() == o.getClass() && this.value == ((Enumerated)o).value;
    }
    
    @Override
    public int hashCode() {
        return (this.getClass().getName() + String.valueOf(this.value)).hashCode();
    }
    
    @Override
    public String toString() {
        return (String)this.getIntTable().get(new Integer(this.value));
    }
    
    protected abstract Hashtable<Integer, String> getIntTable();
    
    protected abstract Hashtable<String, Integer> getStringTable();
}
